/* cursorops.h
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2001-2004,2009 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef CURSOROPS_H
#define CURSOROPS_H

#include "wdefines.h"
#include "functionproto.h"
class Worker;

class RowUp:public FunctionProto
{
public:
  RowUp();
  virtual ~RowUp();
  RowUp( const RowUp &other );
  RowUp &operator=( const RowUp &other );

  virtual RowUp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class RowDown:public FunctionProto
{
public:
  RowDown();
  virtual ~RowDown();
  RowDown( const RowDown &other );
  RowDown &operator=( const RowDown &other );

  virtual RowDown *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class FirstRow:public FunctionProto
{
public:
  FirstRow();
  virtual ~FirstRow();
  FirstRow( const FirstRow &other );
  FirstRow &operator=( const FirstRow &other );

  virtual FirstRow *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class LastRow:public FunctionProto
{
public:
  LastRow();
  virtual ~LastRow();
  LastRow( const LastRow &other );
  LastRow &operator=( const LastRow &other );

  virtual LastRow *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class PageUp:public FunctionProto
{
public:
  PageUp();
  virtual ~PageUp();
  PageUp( const PageUp &other );
  PageUp &operator=( const PageUp &other );

  virtual PageUp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class PageDown:public FunctionProto
{
public:
  PageDown();
  virtual ~PageDown();
  PageDown( const PageDown &other );
  PageDown &operator=( const PageDown &other );

  virtual PageDown *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class SelectOp:public FunctionProto
{
public:
  SelectOp();
  virtual ~SelectOp();
  SelectOp( const SelectOp &other );
  SelectOp &operator=( const SelectOp &other );

  virtual SelectOp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class SelectAllOp:public FunctionProto
{
public:
  SelectAllOp();
  virtual ~SelectAllOp();
  SelectAllOp( const SelectAllOp &other );
  SelectAllOp &operator=( const SelectAllOp &other );

  virtual SelectAllOp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class SelectNoneOp:public FunctionProto
{
public:
  SelectNoneOp();
  virtual ~SelectNoneOp();
  SelectNoneOp( const SelectNoneOp &other );
  SelectNoneOp &operator=( const SelectNoneOp &other );

  virtual SelectNoneOp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class InvertAllOp:public FunctionProto
{
public:
  InvertAllOp();
  virtual ~InvertAllOp();
  InvertAllOp( const InvertAllOp &other );
  InvertAllOp &operator=( const InvertAllOp &other );

  virtual InvertAllOp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

class ParentDirOp:public FunctionProto
{
public:
  ParentDirOp();
  virtual ~ParentDirOp();
  ParentDirOp( const ParentDirOp &other );
  ParentDirOp &operator=( const ParentDirOp &other );

  virtual ParentDirOp *duplicate() const;
  virtual bool isName(const char *);
  virtual const char *getName();
#if ! ( defined ( __GNUC__ ) && ( __GNUC__ < 3 ) )
  using FunctionProto::run;
#endif
  int run( WPUContext *wpu, ActionMessage* );
  virtual const char *getDescription();
protected:
  static const char *name;
};

#endif

/* Local Variables: */
/* mode:c++ */
/* End: */
