/* hw_volume_manager.hh
 * This file belongs to Worker, a file manager for UN*X/X11.
 * Copyright (C) 2009 Ralf Hoffmann.
 * You can contact me at: ralf@boomerangsworld.de
 *   or http://www.boomerangsworld.de/worker
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 */

#ifndef HW_VOLUME_MANAGER_HH
#define HW_VOLUME_MANAGER_HH

#include "wdefines.h"
#include <string>
#include <list>
#include <set>

//TODO put includes into wdefines?
#ifdef HAVE_HAL_DBUS
#include <hal/libhal.h>
#include <dbus/dbus.h>
#include <dbus/dbus-glib.h>
#endif

#include "fstabmounttable.hh"
#include "mtabmounttable.hh"

class HWVolume;

class HWVolumeManager
{
public:
    HWVolumeManager( const std::string &fstab_location = HW_VM_FSTAB,
                     const std::string &mtab_location = HW_VM_MTAB );
    ~HWVolumeManager();
    
    std::list<HWVolume> getVolumes();
    int mount( HWVolume &vol, std::string &error_return );
    int umount( HWVolume &vol, std::string &error_return );

    void update( HWVolume &vol );
    std::list<HWVolume> getNewVolumes( bool store_new = true );
    std::list<HWVolume> filterUniqueDevices( const std::list<HWVolume> &l ) const;

    void setMountCommand( const std::string &cmd );
    void setUnmountCommand( const std::string &cmd );
    void setPartitionFile( const std::string &file );
    void setFStabFile( const std::string &file );
    void setMtabFile( const std::string &file );
private:
#ifdef HAVE_HAL_DBUS
    DBusConnection *m_dbus_conn;
    LibHalContext *m_hal_ctx;
#endif
    std::list< std::string > m_avail_parts;
    std::set< std::string > m_known_devices;
    FStabMountTable m_fstab_table;
    MtabMountTable m_mtab_table;
    std::string m_mount_command, m_unmount_command;
    std::string m_partition_file;

    std::list< std::string > getAvailablePartitions();
    bool isAvailablePartition( const std::string &dev,
                               const std::list< std::string > &l );
    bool isSameDevice( const std::string &dev1,
                       const std::string &dev2 );
    bool isDevice( const std::string &dev );

    bool getBoolProperty( const std::string &udi,
                          const std::string &property );
    std::string getStringProperty( const std::string &udi,
                                   const std::string &property );
    std::list< std::string > getStrlistProperty( const std::string &udi,
                                                 const std::string &property );

    int halMount( HWVolume &vol, std::string &error_return );
    int cmdMount( HWVolume &vol, std::string &error_return );
    int halUnmount( HWVolume &vol, std::string &error_return );
    int cmdUnmount( HWVolume &vol, std::string &error_return );

    int HALconnectionInit();
    int HALconnectionFini();
    bool HALisConnected();
    int HALreconnect();
};

#endif
